@@ -1,14 +1,19 @@
 from email.utils import formatdate
 import json
 
-from testing.http_util import ChunkedResponse, JsonResponse
+from ldclient.impl.model import ModelEntity
 from ldclient.interfaces import EventProcessor, FeatureRequester, FeatureStore, UpdateProcessor
 
+from testing.http_util import ChunkedResponse, JsonResponse
+
+
+def item_as_json(item):
+    return item.to_json_dict() if isinstance(item, ModelEntity) else item
 
 def make_items_map(items = []):
     ret = {}
     for item in items:
-        ret[item['key']] = item
+        ret[item['key']] = item_as_json(item)
     return ret
 
 def make_put_event(flags = [], segments = []):
@@ -17,7 +22,7 @@ def make_put_event(flags = [], segments = []):
 
 def make_patch_event(kind, item):
     path = '%s%s' % (kind.stream_api_path, item['key'])
-    data = { "path": path, "data": item }
+    data = { "path": path, "data": item_as_json(item) }
     return 'event:patch\ndata: %s\n\n' % json.dumps(data)
 
 def make_delete_event(kind, key, version):
